@@ -180,6 +180,10 @@ tgz = project.repository_archive(sha='4567abc')
 contributors = project.repository_contributors()
 # end repository contributors
 
+# housekeeping
+project.housekeeping()
+# end housekeeping
+
 # files get
 f = project.files.get(file_path='README.rst', ref='master')
 
@@ -206,7 +210,7 @@ f = project.files.create({'file_path': 'testfile',
 
 # files update
 f.content = 'new content'
-f.save(branch'master', commit_message='Update testfile')  # v4
+f.save(branch='master', commit_message='Update testfile')  # v4
 f.save(branch_name='master', commit_message='Update testfile')  # v3
 
 # or for binary data
@@ -301,8 +305,11 @@ note.delete()
 # end notes delete
 
 # service get
+# For v3
 service = project.services.get(service_name='asana', project_id=1)
-# display it's status (enabled/disabled)
+# For v4
+service = project.services.get('asana')
+# display its status (enabled/disabled)
 print(service.active)
 # end service get
 
