@@ -34,7 +34,7 @@ from gitlab.exceptions import *  # noqa
 from gitlab.v3.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '1.1.0'
+__version__ = '1.2.0'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -59,6 +59,7 @@ class Gitlab(object):
     Args:
         url (str): The URL of the GitLab server.
         private_token (str): The user private token
+        oauth_token (str): An oauth token
         email (str): The user email or login.
         password (str): The user password (associated with email).
         ssl_verify (bool|str): Whether SSL certificates should be validated. If
@@ -70,9 +71,10 @@ class Gitlab(object):
         api_version (str): Gitlab API version to use (3 or 4)
     """
 
-    def __init__(self, url, private_token=None, email=None, password=None,
-                 ssl_verify=True, http_username=None, http_password=None,
-                 timeout=None, api_version='3', session=None):
+    def __init__(self, url, private_token=None, oauth_token=None, email=None,
+                 password=None, ssl_verify=True, http_username=None,
+                 http_password=None, timeout=None, api_version='3',
+                 session=None):
 
         self._api_version = str(api_version)
         self._server_version = self._server_revision = None
@@ -81,15 +83,19 @@ class Gitlab(object):
         self.timeout = timeout
         #: Headers that will be used in request to GitLab
         self.headers = {}
-        self._set_token(private_token)
+
         #: The user email
         self.email = email
         #: The user password (associated with email)
         self.password = password
         #: Whether SSL certificates should be validated
         self.ssl_verify = ssl_verify
+
+        self.private_token = private_token
         self.http_username = http_username
         self.http_password = http_password
+        self.oauth_token = oauth_token
+        self._set_auth_info()
 
         #: Create a session object for requests
         self.session = session or requests.Session()
@@ -116,10 +122,12 @@ class Gitlab(object):
         self.users = objects.UserManager(self)
         self.todos = objects.TodoManager(self)
         if self._api_version == '3':
-            self.keys = objects.KeyManager(self)
             self.teams = objects.TeamManager(self)
         else:
             self.dockerfiles = objects.DockerfileManager(self)
+            self.features = objects.FeatureManager(self)
+            self.pagesdomains = objects.PagesDomainManager(self)
+            self.user_activities = objects.UserActivitiesManager(self)
 
         if self._api_version == '3':
             # build the "submanagers"
@@ -177,7 +185,8 @@ class Gitlab(object):
         """
         config = gitlab.config.GitlabConfigParser(gitlab_id=gitlab_id,
                                                   config_files=config_files)
-        return Gitlab(config.url, private_token=config.token,
+        return Gitlab(config.url, private_token=config.private_token,
+                      oauth_token=config.oauth_token,
                       ssl_verify=config.ssl_verify, timeout=config.timeout,
                       http_username=config.http_username,
                       http_password=config.http_password,
@@ -191,21 +200,12 @@ class Gitlab(object):
         The `user` attribute will hold a `gitlab.objects.CurrentUser` object on
         success.
         """
-        if self.private_token:
+        if self.private_token or self.oauth_token:
             self._token_auth()
         else:
             self._credentials_auth()
 
-    def credentials_auth(self):
-        """Performs an authentication using email/password."""
-        warnings.warn('credentials_auth() is deprecated and will be removed.',
-                      DeprecationWarning)
-        self._credentials_auth()
-
     def _credentials_auth(self):
-        if not self.email or not self.password:
-            raise GitlabAuthenticationError("Missing email/password")
-
         data = {'email': self.email, 'password': self.password}
         if self.api_version == '3':
             r = self._raw_post('/session', json.dumps(data),
@@ -216,14 +216,8 @@ class Gitlab(object):
             r = self.http_post('/session', data)
             manager = self._objects.CurrentUserManager(self)
             self.user = self._objects.CurrentUser(manager, r)
-
-        self._set_token(self.user.private_token)
-
-    def token_auth(self):
-        """Performs an authentication using the private token."""
-        warnings.warn('token_auth() is deprecated and will be removed.',
-                      DeprecationWarning)
-        self._token_auth()
+        self.private_token = self.user.private_token
+        self._set_auth_info()
 
     def _token_auth(self):
         if self.api_version == '3':
@@ -254,17 +248,6 @@ class Gitlab(object):
 
         return self._server_version, self._server_revision
 
-    def set_url(self, url):
-        """Updates the GitLab URL.
-
-        Args:
-            url (str): Base URL of the GitLab server.
-        """
-        warnings.warn('set_url() is deprecated, create a new Gitlab instance '
-                      'if you need an updated URL.',
-                      DeprecationWarning)
-        self._url = '%s/api/v%s' % (url, self._api_version)
-
     def _construct_url(self, id_, obj, parameters, action=None):
         if 'next_url' in parameters:
             return parameters['next_url']
@@ -289,36 +272,30 @@ class Gitlab(object):
         else:
             return url
 
-    def set_token(self, token):
-        """Sets the private token for authentication.
+    def _set_auth_info(self):
+        if self.private_token and self.oauth_token:
+            raise ValueError("Only one of private_token or oauth_token should "
+                             "be defined")
+        if ((self.http_username and not self.http_password)
+           or (not self.http_username and self.http_password)):
+            raise ValueError("Both http_username and http_password should "
+                             "be defined")
+        if self.oauth_token and self.http_username:
+            raise ValueError("Only one of oauth authentication or http "
+                             "authentication should be defined")
 
-        Args:
-            token (str): The private token.
-        """
-        warnings.warn('set_token() is deprecated, use the private_token '
-                      'argument of the Gitlab constructor.',
-                      DeprecationWarning)
-        self._set_token(token)
+        self._http_auth = None
+        if self.private_token:
+            self.headers['PRIVATE-TOKEN'] = self.private_token
+            self.headers.pop('Authorization', None)
 
-    def _set_token(self, token):
-        self.private_token = token if token else None
-        if token:
-            self.headers["PRIVATE-TOKEN"] = token
-        elif "PRIVATE-TOKEN" in self.headers:
-            del self.headers["PRIVATE-TOKEN"]
+        if self.oauth_token:
+            self.headers['Authorization'] = "Bearer %s" % self.oauth_token
+            self.headers.pop('PRIVATE-TOKEN', None)
 
-    def set_credentials(self, email, password):
-        """Sets the email/login and password for authentication.
-
-        Args:
-            email (str): The user email or login.
-            password (str): The user password.
-        """
-        warnings.warn('set_credentials() is deprecated, use the email and '
-                      'password arguments of the Gitlab constructor.',
-                      DeprecationWarning)
-        self.email = email
-        self.password = password
+        if self.http_username:
+            self._http_auth = requests.auth.HTTPBasicAuth(self.http_username,
+                                                          self.http_password)
 
     def enable_debug(self):
         import logging
@@ -340,16 +317,10 @@ class Gitlab(object):
             request_headers['Content-type'] = content_type
         return request_headers
 
-    def _create_auth(self):
-        if self.http_username and self.http_password:
-            return requests.auth.HTTPBasicAuth(self.http_username,
-                                               self.http_password)
-        return None
-
     def _get_session_opts(self, content_type):
         return {
             'headers': self._create_headers(content_type),
-            'auth': self._create_auth(),
+            'auth': self._http_auth,
             'timeout': self.timeout,
             'verify': self.ssl_verify
         }
@@ -671,8 +642,22 @@ class Gitlab(object):
             return parsed._replace(path=new_path).geturl()
 
         url = self._build_url(path)
-        params = query_data.copy()
-        params.update(kwargs)
+
+        def copy_dict(dest, src):
+            for k, v in src.items():
+                if isinstance(v, dict):
+                    # Transform dict values in new attributes. For example:
+                    # custom_attributes: {'foo', 'bar'} =>
+                    #   custom_attributes['foo']: 'bar'
+                    for dict_k, dict_v in v.items():
+                        dest['%s[%s]' % (k, dict_k)] = dict_v
+                else:
+                    dest[k] = v
+
+        params = {}
+        copy_dict(params, query_data)
+        copy_dict(params, kwargs)
+
         opts = self._get_session_opts(content_type='application/json')
 
         # don't set the content-type header when uploading files
@@ -692,8 +677,9 @@ class Gitlab(object):
                                files=files, **opts)
         prepped = self.session.prepare_request(req)
         prepped.url = sanitized_url(prepped.url)
-        result = self.session.send(prepped, stream=streamed, verify=verify,
-                                   timeout=timeout)
+        settings = self.session.merge_environment_settings(
+            prepped.url, {}, streamed, verify, None)
+        result = self.session.send(prepped, timeout=timeout, **settings)
 
         if 200 <= result.status_code < 300:
             return result
@@ -774,7 +760,7 @@ class Gitlab(object):
         if get_all is True:
             return list(GitlabList(self, url, query_data, **kwargs))
 
-        if 'page' in kwargs or 'per_page' in kwargs or as_list is True:
+        if 'page' in kwargs or as_list is True:
             # pagination requested, we return a list
             return list(GitlabList(self, url, query_data, get_next=False,
                                    **kwargs))
