@@ -77,6 +77,10 @@ class GitlabDeleteError(GitlabOperationError):
     pass
 
 
+class GitlabSetError(GitlabOperationError):
+    pass
+
+
 class GitlabProtectError(GitlabOperationError):
     pass
 
@@ -185,6 +189,10 @@ class GitlabCherryPickError(GitlabOperationError):
     pass
 
 
+class GitlabHousekeepingError(GitlabOperationError):
+    pass
+
+
 def raise_error_from_response(response, error, expected_code=200):
     """Tries to parse gitlab error message from response and raises error.
 
