@@ -112,6 +112,26 @@ class SidekiqManager(RESTManager):
         return self.gitlab.http_get('/sidekiq/compound_metrics', **kwargs)
 
 
+class UserActivities(RESTObject):
+    _id_attr = 'username'
+
+
+class UserActivitiesManager(ListMixin, RESTManager):
+    _path = '/user/activities'
+    _obj_cls = UserActivities
+
+
+class UserCustomAttribute(ObjectDeleteMixin, RESTObject):
+    _id_attr = 'key'
+
+
+class UserCustomAttributeManager(RetrieveMixin, SetMixin, DeleteMixin,
+                                 RESTManager):
+    _path = '/users/%(user_id)s/custom_attributes'
+    _obj_cls = UserCustomAttribute
+    _from_parent_attrs = {'user_id': 'id'}
+
+
 class UserEmail(ObjectDeleteMixin, RESTObject):
     _short_print_attr = 'email'
 
@@ -145,6 +165,18 @@ class UserKeyManager(GetFromListMixin, CreateMixin, DeleteMixin, RESTManager):
     _create_attrs = (('title', 'key'), tuple())
 
 
+class UserImpersonationToken(ObjectDeleteMixin, RESTObject):
+    pass
+
+
+class UserImpersonationTokenManager(NoUpdateMixin, RESTManager):
+    _path = '/users/%(user_id)s/impersonation_tokens'
+    _obj_cls = UserImpersonationToken
+    _from_parent_attrs = {'user_id': 'id'}
+    _create_attrs = (('name', 'scopes'), ('expires_at',))
+    _list_filters = ('state',)
+
+
 class UserProject(RESTObject):
     pass
 
@@ -165,8 +197,10 @@ class UserProjectManager(CreateMixin, RESTManager):
 class User(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = 'username'
     _managers = (
+        ('customattributes', 'UserCustomAttributeManager'),
         ('emails', 'UserEmailManager'),
         ('gpgkeys', 'UserGPGKeyManager'),
+        ('impersonationtokens', 'UserImpersonationTokenManager'),
         ('keys', 'UserKeyManager'),
         ('projects', 'UserProjectManager'),
     )
@@ -219,7 +253,7 @@ class UserManager(CRUDMixin, RESTManager):
     _obj_cls = User
 
     _list_filters = ('active', 'blocked', 'username', 'extern_uid', 'provider',
-                     'external', 'search')
+                     'external', 'search', 'custom_attributes')
     _create_attrs = (
         tuple(),
         ('email', 'username', 'name', 'password', 'reset_password', 'skype',
@@ -366,6 +400,38 @@ class DockerfileManager(RetrieveMixin, RESTManager):
     _obj_cls = Dockerfile
 
 
+class Feature(RESTObject):
+    _id_attr = 'name'
+
+
+class FeatureManager(ListMixin, RESTManager):
+    _path = '/features/'
+    _obj_cls = Feature
+
+    @exc.on_http_error(exc.GitlabSetError)
+    def set(self, name, value, feature_group=None, user=None, **kwargs):
+        """Create or update the object.
+
+        Args:
+            name (str): The value to set for the object
+            value (bool/int): The value to set for the object
+            feature_group (str): A feature group name
+            user (str): A GitLab username
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabSetError: If an error occured
+
+        Returns:
+            obj: The created/updated attribute
+        """
+        path = '%s/%s' % (self.path, name.replace('/', '%2F'))
+        data = {'value': value, 'feature_group': feature_group, 'user': user}
+        server_data = self.gitlab.http_post(path, post_data=data, **kwargs)
+        return self._obj_cls(self, server_data)
+
+
 class Gitignore(RESTObject):
     _id_attr = 'name'
 
@@ -395,6 +461,17 @@ class GroupAccessRequestManager(GetFromListMixin, CreateMixin, DeleteMixin,
     _from_parent_attrs = {'group_id': 'id'}
 
 
+class GroupCustomAttribute(ObjectDeleteMixin, RESTObject):
+    _id_attr = 'key'
+
+
+class GroupCustomAttributeManager(RetrieveMixin, SetMixin, DeleteMixin,
+                                  RESTManager):
+    _path = '/groups/%(group_id)s/custom_attributes'
+    _obj_cls = GroupCustomAttribute
+    _from_parent_attrs = {'group_id': 'id'}
+
+
 class GroupIssue(RESTObject):
     pass
 
@@ -436,6 +513,11 @@ class GroupMilestone(SaveMixin, ObjectDeleteMixin, RESTObject):
         """List issues related to this milestone.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -460,6 +542,11 @@ class GroupMilestone(SaveMixin, ObjectDeleteMixin, RESTObject):
         """List the merge requests related to this milestone.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -510,6 +597,18 @@ class GroupProjectManager(GetFromListMixin, RESTManager):
                      'ci_enabled_first')
 
 
+class GroupSubgroup(RESTObject):
+    pass
+
+
+class GroupSubgroupManager(GetFromListMixin, RESTManager):
+    _path = '/groups/%(group_id)s/subgroups'
+    _obj_cls = GroupSubgroup
+    _from_parent_attrs = {'group_id': 'id'}
+    _list_filters = ('skip_groups', 'all_available', 'search', 'order_by',
+                     'sort', 'statistics', 'owned')
+
+
 class GroupVariable(SaveMixin, ObjectDeleteMixin, RESTObject):
     _id_attr = 'key'
 
@@ -526,11 +625,13 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = 'name'
     _managers = (
         ('accessrequests', 'GroupAccessRequestManager'),
+        ('customattributes', 'GroupCustomAttributeManager'),
+        ('issues', 'GroupIssueManager'),
         ('members', 'GroupMemberManager'),
         ('milestones', 'GroupMilestoneManager'),
         ('notificationsettings', 'GroupNotificationSettingsManager'),
         ('projects', 'GroupProjectManager'),
-        ('issues', 'GroupIssueManager'),
+        ('subgroups', 'GroupSubgroupManager'),
         ('variables', 'GroupVariableManager'),
     )
 
@@ -554,6 +655,8 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
 class GroupManager(CRUDMixin, RESTManager):
     _path = '/groups'
     _obj_cls = Group
+    _list_filters = ('skip_groups', 'all_available', 'search', 'order_by',
+                     'sort', 'statistics', 'owned', 'custom_attributes')
     _create_attrs = (
         ('name', 'path'),
         ('description', 'visibility', 'parent_id', 'lfs_enabled',
@@ -664,6 +767,15 @@ class NamespaceManager(GetFromListMixin, RESTManager):
     _list_filters = ('search', )
 
 
+class PagesDomain(RESTObject):
+    _id_attr = 'domain'
+
+
+class PagesDomainManager(ListMixin, RESTManager):
+    _path = '/pages/domains'
+    _obj_cls = PagesDomain
+
+
 class ProjectBoardList(SaveMixin, ObjectDeleteMixin, RESTObject):
     pass
 
@@ -739,6 +851,17 @@ class ProjectBranchManager(NoUpdateMixin, RESTManager):
     _create_attrs = (('branch', 'ref'), tuple())
 
 
+class ProjectCustomAttribute(ObjectDeleteMixin, RESTObject):
+    _id_attr = 'key'
+
+
+class ProjectCustomAttributeManager(RetrieveMixin, SetMixin, DeleteMixin,
+                                    RESTManager):
+    _path = '/projects/%(project_id)s/custom_attributes'
+    _obj_cls = ProjectCustomAttribute
+    _from_parent_attrs = {'project_id': 'id'}
+
+
 class ProjectJob(RESTObject):
     @cli.register_custom_action('ProjectJob')
     @exc.on_http_error(exc.GitlabJobCancelError)
@@ -1057,10 +1180,35 @@ class ProjectHookManager(CRUDMixin, RESTManager):
     )
 
 
-class ProjectIssueNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+class ProjectIssueAwardEmoji(ObjectDeleteMixin, RESTObject):
     pass
 
 
+class ProjectIssueAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = '/projects/%(project_id)s/issues/%(issue_iid)s/award_emoji'
+    _obj_cls = ProjectIssueAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id', 'issue_iid': 'iid'}
+    _create_attrs = (('name', ), tuple())
+
+
+class ProjectIssueNoteAwardEmoji(ObjectDeleteMixin, RESTObject):
+    pass
+
+
+class ProjectIssueNoteAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = ('/projects/%(project_id)s/issues/%(issue_iid)s'
+             '/notes/%(note_id)s/award_emoji')
+    _obj_cls = ProjectIssueNoteAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id',
+                          'issue_iid': 'issue_iid',
+                          'note_id': 'id'}
+    _create_attrs = (('name', ), tuple())
+
+
+class ProjectIssueNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+    _managers = (('awardemojis', 'ProjectIssueNoteAwardEmojiManager'),)
+
+
 class ProjectIssueNoteManager(CRUDMixin, RESTManager):
     _path = '/projects/%(project_id)s/issues/%(issue_iid)s/notes'
     _obj_cls = ProjectIssueNote
@@ -1073,7 +1221,25 @@ class ProjectIssue(SubscribableMixin, TodoMixin, TimeTrackingMixin, SaveMixin,
                    ObjectDeleteMixin, RESTObject):
     _short_print_attr = 'title'
     _id_attr = 'iid'
-    _managers = (('notes', 'ProjectIssueNoteManager'), )
+    _managers = (
+        ('notes', 'ProjectIssueNoteManager'),
+        ('awardemojis', 'ProjectIssueAwardEmojiManager'),
+    )
+
+    @cli.register_custom_action('ProjectIssue')
+    @exc.on_http_error(exc.GitlabUpdateError)
+    def user_agent_detail(self, **kwargs):
+        """Get user agent detail.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabGetError: If the detail could not be retrieved
+        """
+        path = '%s/%s/user_agent_detail' % (self.manager.path, self.get_id())
+        return self.manager.gitlab.http_get(path, **kwargs)
 
     @cli.register_custom_action('ProjectIssue', ('to_project_id',))
     @exc.on_http_error(exc.GitlabUpdateError)
@@ -1147,6 +1313,18 @@ class ProjectNotificationSettingsManager(NotificationSettingsManager):
     _from_parent_attrs = {'project_id': 'id'}
 
 
+class ProjectPagesDomain(SaveMixin, ObjectDeleteMixin, RESTObject):
+    _id_attr = 'domain'
+
+
+class ProjectPagesDomainManager(CRUDMixin, RESTManager):
+    _path = '/projects/%(project_id)s/pages/domains'
+    _obj_cls = ProjectPagesDomain
+    _from_parent_attrs = {'project_id': 'id'}
+    _create_attrs = (('domain', ), ('certificate', 'key'))
+    _update_attrs = (tuple(), ('certificate', 'key'))
+
+
 class ProjectTag(ObjectDeleteMixin, RESTObject):
     _id_attr = 'name'
     _short_print_attr = 'name'
@@ -1194,6 +1372,17 @@ class ProjectTagManager(NoUpdateMixin, RESTManager):
     _create_attrs = (('tag_name', 'ref'), ('message',))
 
 
+class ProjectMergeRequestAwardEmoji(ObjectDeleteMixin, RESTObject):
+    pass
+
+
+class ProjectMergeRequestAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = '/projects/%(project_id)s/merge_requests/%(mr_iid)s/award_emoji'
+    _obj_cls = ProjectMergeRequestAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id', 'mr_iid': 'iid'}
+    _create_attrs = (('name', ), tuple())
+
+
 class ProjectMergeRequestDiff(RESTObject):
     pass
 
@@ -1204,10 +1393,24 @@ class ProjectMergeRequestDiffManager(RetrieveMixin, RESTManager):
     _from_parent_attrs = {'project_id': 'project_id', 'mr_iid': 'iid'}
 
 
-class ProjectMergeRequestNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+class ProjectMergeRequestNoteAwardEmoji(ObjectDeleteMixin, RESTObject):
     pass
 
 
+class ProjectMergeRequestNoteAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = ('/projects/%(project_id)s/merge_requests/%(mr_iid)s'
+             '/notes/%(note_id)s/award_emoji')
+    _obj_cls = ProjectMergeRequestNoteAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id',
+                          'mr_iid': 'issue_iid',
+                          'note_id': 'id'}
+    _create_attrs = (('name', ), tuple())
+
+
+class ProjectMergeRequestNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+    _managers = (('awardemojis', 'ProjectMergeRequestNoteAwardEmojiManager'),)
+
+
 class ProjectMergeRequestNoteManager(CRUDMixin, RESTManager):
     _path = '/projects/%(project_id)s/merge_requests/%(mr_iid)s/notes'
     _obj_cls = ProjectMergeRequestNote
@@ -1221,8 +1424,9 @@ class ProjectMergeRequest(SubscribableMixin, TodoMixin, TimeTrackingMixin,
     _id_attr = 'iid'
 
     _managers = (
+        ('awardemojis', 'ProjectMergeRequestAwardEmojiManager'),
+        ('diffs', 'ProjectMergeRequestDiffManager'),
         ('notes', 'ProjectMergeRequestNoteManager'),
-        ('diffs', 'ProjectMergeRequestDiffManager')
     )
 
     @cli.register_custom_action('ProjectMergeRequest')
@@ -1250,6 +1454,11 @@ class ProjectMergeRequest(SubscribableMixin, TodoMixin, TimeTrackingMixin,
         """List issues that will close on merge."
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -1272,6 +1481,11 @@ class ProjectMergeRequest(SubscribableMixin, TodoMixin, TimeTrackingMixin,
         """List the merge request commits.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -1374,6 +1588,11 @@ class ProjectMilestone(SaveMixin, ObjectDeleteMixin, RESTObject):
         """List issues related to this milestone.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -1398,6 +1617,11 @@ class ProjectMilestone(SaveMixin, ObjectDeleteMixin, RESTObject):
         """List the merge requests related to this milestone.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -1576,9 +1800,10 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin,
         """
 
         self._check_missing_create_attrs(data)
-        file_path = data.pop('file_path').replace('/', '%2F')
+        new_data = data.copy()
+        file_path = new_data.pop('file_path').replace('/', '%2F')
         path = '%s/%s' % (self.path, file_path)
-        server_data = self.gitlab.http_post(path, post_data=data, **kwargs)
+        server_data = self.gitlab.http_post(path, post_data=new_data, **kwargs)
         return self._obj_cls(self, server_data)
 
     @exc.on_http_error(exc.GitlabUpdateError)
@@ -1715,10 +1940,24 @@ class ProjectPipelineManager(RetrieveMixin, CreateMixin, RESTManager):
         return CreateMixin.create(self, data, path=path, **kwargs)
 
 
-class ProjectSnippetNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+class ProjectSnippetNoteAwardEmoji(ObjectDeleteMixin, RESTObject):
     pass
 
 
+class ProjectSnippetNoteAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = ('/projects/%(project_id)s/snippets/%(snippet_id)s'
+             '/notes/%(note_id)s/award_emoji')
+    _obj_cls = ProjectSnippetNoteAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id',
+                          'snippet_id': 'snippet_id',
+                          'note_id': 'id'}
+    _create_attrs = (('name', ), tuple())
+
+
+class ProjectSnippetNote(SaveMixin, ObjectDeleteMixin, RESTObject):
+    _managers = (('awardemojis', 'ProjectSnippetNoteAwardEmojiManager'),)
+
+
 class ProjectSnippetNoteManager(CRUDMixin, RESTManager):
     _path = '/projects/%(project_id)s/snippets/%(snippet_id)s/notes'
     _obj_cls = ProjectSnippetNote
@@ -1728,10 +1967,24 @@ class ProjectSnippetNoteManager(CRUDMixin, RESTManager):
     _update_attrs = (('body', ), tuple())
 
 
+class ProjectSnippetAwardEmoji(ObjectDeleteMixin, RESTObject):
+    pass
+
+
+class ProjectSnippetAwardEmojiManager(NoUpdateMixin, RESTManager):
+    _path = '/projects/%(project_id)s/snippets/%(snippet_id)s/award_emoji'
+    _obj_cls = ProjectSnippetAwardEmoji
+    _from_parent_attrs = {'project_id': 'project_id', 'snippet_id': 'id'}
+    _create_attrs = (('name', ), tuple())
+
+
 class ProjectSnippet(SaveMixin, ObjectDeleteMixin, RESTObject):
     _url = '/projects/%(project_id)s/snippets'
     _short_print_attr = 'title'
-    _managers = (('notes', 'ProjectSnippetNoteManager'), )
+    _managers = (
+        ('awardemojis', 'ProjectSnippetAwardEmojiManager'),
+        ('notes', 'ProjectSnippetNoteManager'),
+    )
 
     @cli.register_custom_action('ProjectSnippet')
     @exc.on_http_error(exc.GitlabGetError)
@@ -1846,6 +2099,7 @@ class ProjectServiceManager(GetMixin, UpdateMixin, DeleteMixin, RESTManager):
                  ('new_issue_url', 'project_url', 'issues_url', 'api_url',
                   'description', 'username', 'password',
                   'jira_issue_transition_id')),
+        'mattermost': (('webhook',), ('username', 'channel')),
         'pivotaltracker': (('token', ), tuple()),
         'pushover': (('api_key', 'user_key', 'priority'), ('device', 'sound')),
         'redmine': (('new_issue_url', 'project_url', 'issues_url'),
@@ -1969,6 +2223,7 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         ('branches', 'ProjectBranchManager'),
         ('jobs', 'ProjectJobManager'),
         ('commits', 'ProjectCommitManager'),
+        ('customattributes', 'ProjectCustomAttributeManager'),
         ('deployments', 'ProjectDeploymentManager'),
         ('environments', 'ProjectEnvironmentManager'),
         ('events', 'ProjectEventManager'),
@@ -1983,6 +2238,7 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         ('milestones', 'ProjectMilestoneManager'),
         ('notes', 'ProjectNoteManager'),
         ('notificationsettings', 'ProjectNotificationSettingsManager'),
+        ('pagesdomains', 'ProjectPagesDomainManager'),
         ('pipelines', 'ProjectPipelineManager'),
         ('protectedbranches', 'ProjectProtectedBranchManager'),
         ('runners', 'ProjectRunnerManager'),
@@ -2003,6 +2259,11 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         Args:
             path (str): Path of the top folder (/ by default)
             ref (str): Reference to a commit or branch
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -2097,6 +2358,11 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         """Return a list of contributors for the project.
 
         Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -2278,6 +2544,22 @@ class Project(SaveMixin, ObjectDeleteMixin, RESTObject):
         post_data = {'ref': ref, 'token': token, 'variables': variables}
         self.manager.gitlab.http_post(path, post_data=post_data, **kwargs)
 
+    @cli.register_custom_action('Project')
+    @exc.on_http_error(exc.GitlabHousekeepingError)
+    def housekeeping(self, **kwargs):
+        """Start the housekeeping task.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabHousekeepingError: If the server failed to perform the
+                                     request
+        """
+        path = '/projects/%s/housekeeping' % self.get_id()
+        self.manager.gitlab.http_post(path, **kwargs)
+
     # see #56 - add file attachment features
     @cli.register_custom_action('Project', ('filename', 'filepath'))
     @exc.on_http_error(exc.GitlabUploadError)
@@ -2357,7 +2639,8 @@ class ProjectManager(CRUDMixin, RESTManager):
     )
     _list_filters = ('search', 'owned', 'starred', 'archived', 'visibility',
                      'order_by', 'sort', 'simple', 'membership', 'statistics',
-                     'with_issues_enabled', 'with_merge_requests_enabled')
+                     'with_issues_enabled', 'with_merge_requests_enabled',
+                     'custom_attributes')
 
 
 class Runner(SaveMixin, ObjectDeleteMixin, RESTObject):
