@@ -51,14 +51,8 @@ qG2ZdhHHmSK2LaQLFiSprUkikStNU9BqSQ==
 -----END PGP PUBLIC KEY BLOCK-----'''
 
 
-# login/password authentication
-gl = gitlab.Gitlab('http://localhost:8080', email=LOGIN, password=PASSWORD)
-gl.auth()
-token_from_auth = gl.private_token
-
 # token authentication from config file
 gl = gitlab.Gitlab.from_config(config_files=['/tmp/python-gitlab.cfg'])
-assert(token_from_auth == gl.private_token)
 gl.auth()
 assert(isinstance(gl.user, gitlab.v4.objects.CurrentUser))
 
@@ -131,6 +125,32 @@ assert(len(new_user.emails.list()) == 1)
 email.delete()
 assert(len(new_user.emails.list()) == 0)
 
+# custom attributes
+attrs = new_user.customattributes.list()
+assert(len(attrs) == 0)
+attr = new_user.customattributes.set('key', 'value1')
+assert(len(gl.users.list(custom_attributes={'key': 'value1'})) == 1)
+assert(attr.key == 'key')
+assert(attr.value == 'value1')
+assert(len(new_user.customattributes.list()) == 1)
+attr = new_user.customattributes.set('key', 'value2')
+attr = new_user.customattributes.get('key')
+assert(attr.value == 'value2')
+assert(len(new_user.customattributes.list()) == 1)
+attr.delete()
+assert(len(new_user.customattributes.list()) == 0)
+
+# impersonation tokens
+user_token = new_user.impersonationtokens.create(
+    {'name': 'token1', 'scopes': ['api', 'read_user']})
+l = new_user.impersonationtokens.list(state='active')
+assert(len(l) == 1)
+user_token.delete()
+l = new_user.impersonationtokens.list(state='active')
+assert(len(l) == 0)
+l = new_user.impersonationtokens.list(state='inactive')
+assert(len(l) == 1)
+
 new_user.delete()
 foobar_user.delete()
 assert(len(gl.users.list()) == 3)
@@ -185,8 +205,9 @@ p_id = gl.groups.list(search='group2')[0].id
 group3 = gl.groups.create({'name': 'group3', 'path': 'group3', 'parent_id': p_id})
 
 assert(len(gl.groups.list()) == 3)
-assert(len(gl.groups.list(search='1')) == 1)
+assert(len(gl.groups.list(search='oup1')) == 1)
 assert(group3.parent_id == p_id)
+assert(group2.subgroups.list()[0].id == group3.id)
 
 group1.members.create({'access_level': gitlab.Group.OWNER_ACCESS,
                        'user_id': user1.id})
@@ -210,6 +231,21 @@ assert(member.access_level == gitlab.Group.OWNER_ACCESS)
 
 group2.members.delete(gl.user.id)
 
+# group custom attributes
+attrs = group2.customattributes.list()
+assert(len(attrs) == 0)
+attr = group2.customattributes.set('key', 'value1')
+assert(len(gl.groups.list(custom_attributes={'key': 'value1'})) == 1)
+assert(attr.key == 'key')
+assert(attr.value == 'value1')
+assert(len(group2.customattributes.list()) == 1)
+attr = group2.customattributes.set('key', 'value2')
+attr = group2.customattributes.get('key')
+assert(attr.value == 'value2')
+assert(len(group2.customattributes.list()) == 1)
+attr.delete()
+assert(len(group2.customattributes.list()) == 0)
+
 # group notification settings
 settings = group2.notificationsettings.get()
 settings.level = 'disabled'
@@ -265,6 +301,30 @@ assert(len(l1) == 1)
 assert(len(l2) == 1)
 assert(l1[0].id != l2[0].id)
 
+# group custom attributes
+attrs = admin_project.customattributes.list()
+assert(len(attrs) == 0)
+attr = admin_project.customattributes.set('key', 'value1')
+assert(len(gl.projects.list(custom_attributes={'key': 'value1'})) == 1)
+assert(attr.key == 'key')
+assert(attr.value == 'value1')
+assert(len(admin_project.customattributes.list()) == 1)
+attr = admin_project.customattributes.set('key', 'value2')
+attr = admin_project.customattributes.get('key')
+assert(attr.value == 'value2')
+assert(len(admin_project.customattributes.list()) == 1)
+attr.delete()
+assert(len(admin_project.customattributes.list()) == 0)
+
+# project pages domains
+domain = admin_project.pagesdomains.create({'domain': 'foo.domain.com'})
+assert(len(admin_project.pagesdomains.list()) == 1)
+assert(len(gl.pagesdomains.list()) == 1)
+domain = admin_project.pagesdomains.get('foo.domain.com')
+assert(domain.domain == 'foo.domain.com')
+domain.delete()
+assert(len(admin_project.pagesdomains.list()) == 0)
+
 # project content (files)
 admin_project.files.create({'file_path': 'README',
                             'branch': 'master',
@@ -306,6 +366,9 @@ assert(len(commit.statuses.list()) == 1)
 commit.comments.create({'note': 'This is a commit comment'})
 assert(len(commit.comments.list()) == 1)
 
+# housekeeping
+admin_project.housekeeping()
+
 # repository
 tree = admin_project.repository_tree()
 assert(len(tree) != 0)
@@ -408,8 +471,13 @@ assert(len(admin_project.issues.list(milestone='milestone1')) == 1)
 assert(m1.issues().next().title == 'my issue 1')
 note = issue1.notes.create({'body': 'This is an issue note'})
 assert(len(issue1.notes.list()) == 1)
+emoji = note.awardemojis.create({'name': 'tractor'})
+assert(len(note.awardemojis.list()) == 1)
+emoji.delete()
+assert(len(note.awardemojis.list()) == 0)
 note.delete()
 assert(len(issue1.notes.list()) == 0)
+assert(isinstance(issue1.user_agent_detail(), dict))
 
 # tags
 tag1 = admin_project.tags.create({'tag_name': 'v1.0', 'ref': 'master'})
@@ -471,7 +539,7 @@ except gitlab.GitlabMRClosedError:
 p_b = admin_project.protectedbranches.create({'name': '*-stable'})
 assert(p_b.name == '*-stable')
 p_b = admin_project.protectedbranches.get('*-stable')
-# master is protected by default
+# master is protected by default when a branch has been created
 assert(len(admin_project.protectedbranches.list()) == 2)
 admin_project.protectedbranches.delete('master')
 p_b.delete()
@@ -514,6 +582,11 @@ assert(len(ns) != 0)
 ns = gl.namespaces.list(search='root', all=True)[0]
 assert(ns.kind == 'user')
 
+# features
+feat = gl.features.set('foo', 30)
+assert(feat.name == 'foo')
+assert(len(gl.features.list()) == 1)
+
 # broadcast messages
 msg = gl.broadcastmessages.create({'message': 'this is the message'})
 msg.color = '#444444'
@@ -555,3 +628,6 @@ assert(snippet.title == 'updated_title')
 content = snippet.content()
 assert(content == 'import gitlab')
 snippet.delete()
+
+# user activities
+gl.user_activities.list()
